@@ -3,7 +3,7 @@
 # the (default) methods coef, residuals, fitted values can be used             
 ###############################################################################
 
-manyglm <- function (formula, family="negative.binomial", K=1, data=NULL, subset=NULL, na.action=options("na.action"), theta.method = "ML", model = FALSE, x = TRUE, y = TRUE, qr = TRUE, cor.type= "I", shrink.param=NULL, tol=sqrt(.Machine$double.eps), maxiter=25, show.coef=FALSE, show.fitted=FALSE, show.residuals=FALSE, ... ) {
+manyglm <- function (formula, family="negative.binomial", K=1, data=NULL, subset=NULL, na.action=options("na.action"), theta.method = "PHI", model = FALSE, x = TRUE, y = TRUE, qr = TRUE, cor.type= "I", shrink.param=NULL, tol=sqrt(.Machine$double.eps), maxiter=25, maxiter2=10, show.coef=FALSE, show.fitted=FALSE, show.residuals=FALSE, show.warning=FALSE,... ) {
 
 # tasmX <- as.matrix(tasmX, "numeric")  
 
@@ -113,10 +113,14 @@ else {
 # the following values need to be converted to integer types 
     if (theta.method == "ML") methodnum <- 0
     else if (theta.method == "Chi2") methodnum <- 1 
+    else if (theta.method == "PHI") methodnum <- 2 
     else stop("Check the param estimation method name.")  
 
+    if (show.warning==TRUE) warn <- 1
+    else warn <- 0
+
     ######### call Glm Fit Rcpp #########
-    modelParam <- list(tol=tol, regression=familynum, estimation=methodnum, stablizer=FALSE, n=K, maxiter=maxiter)
+    modelParam <- list(tol=tol, regression=familynum, estimation=methodnum, stablizer=FALSE, n=K, maxiter=maxiter, maxiter2=maxiter2, warning=warn)
     z <- .Call("RtoGlm", modelParam, Y, X, PACKAGE="mvabund")
 
 # New codes added for estimating ridge parameter 
@@ -160,6 +164,7 @@ else {
     dimnames(z$stderr.coefficients) <- list(colnames(X), labAbund)
     z$tol <- tol
     z$maxiter <- maxiter
+    z$maxiter2 <- maxiter2
     z$prior.weight <- NULL
     z$AICsum <- sum(z$aic)
     z$family    <- familyname
