@@ -23,6 +23,8 @@ RcppExport SEXP RtoGlmAnova(SEXP mpar, SEXP tpar, SEXP Ysexp, SEXP Xsexp,
     mm.varStab = as<unsigned int>(sparam["stablizer"]);   
     mm.n = as<unsigned int>(sparam["n"]);   
     mm.maxiter = as<unsigned int>(sparam["maxiter"]);   
+    mm.maxiter2 = as<unsigned int>(sparam["maxiter2"]);   
+    mm.warning = as<unsigned int>(sparam["warning"]);   
 
     List rparam(tpar);
     // pass parameters
@@ -34,6 +36,7 @@ RcppExport SEXP RtoGlmAnova(SEXP mpar, SEXP tpar, SEXP Ysexp, SEXP Xsexp,
     tm.reprand = as<unsigned int>(rparam["reprand"]);
     tm.punit = as<unsigned int>(rparam["punit"]);
     tm.showtime = as<unsigned int>(rparam["showtime"]);
+    tm.warning = as<unsigned int>(rparam["warning"]);   
 
 //  for debug
 //    Rprintf("Input param arguments:\n tol=%g, mm.model=%d, nboot=%d, cor_type=%d, shrink_param=%g, test_type=%d, resamp=%d, n=%d, showtime=%d\n", mm.tol, mm.model, tm.nboot, tm.corr, tm.shrink_param, tm.test, tm.resamp, mm.n, tm.showtime);
@@ -141,7 +144,7 @@ RcppExport SEXP RtoGlmAnova(SEXP mpar, SEXP tpar, SEXP Ysexp, SEXP Xsexp,
     unsigned int hours = floor((double)(dif/(double)3600));
     unsigned int min = floor((double)(dif%3600)/(double)60);
     unsigned int sec = dif%60;   
-    if (tm.showtime==TRUE)
+    if (tm.showtime>=TRUE)
         Rprintf("Time elapsed: %d hr %d min %d sec\n", hours, min, sec);
 
     // Wrap the gsl objects with Rcpp 
