@@ -77,7 +77,7 @@
 // estiMethod
 #define NEWTON 0
 #define CHI2 1
-#define FISHER 2
+#define PHI 2
 // infoMatrix
 #define OIM 0
 #define EIM 1
@@ -132,6 +132,7 @@ typedef struct MethodStruc {
     unsigned int nVars;
     unsigned int nParam;
     unsigned int showtime;
+    unsigned int warning;
 
     // numeric
     double shrink_param;
@@ -162,8 +163,9 @@ typedef struct RegressionMethod{
     unsigned int varStab;
     unsigned int estiMethod;
     double tol;
-    unsigned int maxiter;
+    unsigned int maxiter, maxiter2;
     unsigned int n; // used in binomial regression
+    unsigned int warning;
 } reg_Method;
 
 // ManyLM related
@@ -268,7 +270,7 @@ class glm
            unsigned int rdf;
 	   double *theta, *ll, *dev, *aic;
 	   unsigned int *iterconv;  
-           unsigned int maxiter;
+           unsigned int maxiter, maxiter2;
 	   double eps, mintol, maxtol, maxth;
            unsigned int nRows, nVars, nParams;
 //   private: 
@@ -381,9 +383,14 @@ class NBinGlm : public PoissonGlm // Y~NB(n, p)
 	   // size = th (dispersion)
 	   // prob = size/(size+mu)
 	   double weifunc(double mui, double th) const
-	        { return MAX(mui*th, eps)/MAX(mui+th, eps); }
+	        { if (th==0) return 0;
+                  else if (th>maxth) return mui; // poisson
+                  else return MAX(mintol, mui*th/MAX(mui+th, mintol)); }
 	   double varfunc(double mui, double th) const
-	        { return mui+mui*mui/MAX(th, eps); }
+	        { 
+                  if (th==0) return 0;
+                  else if (th>maxth) return mui;
+                  else return mui+mui*mui/MAX(th, mintol); }
 	   double llfunc(double yi, double mui, double th) const;
 	   double devfunc(double yi, double mui, double th) const
 	        { if (th==0) return 0; 
@@ -410,7 +417,7 @@ class NBinGlm : public PoissonGlm // Y~NB(n, p)
 		  // for NB with mean mu and var=mu+mu^2/theta
                   else return Rf_rnbinom(th, th/(mui+th)); }
 
-	   int getfAfAdash (double th, unsigned int id, double *fA, double *fAdash);
+	   double getfAfAdash (double th, unsigned int id, unsigned int limit);
 	   double thetaML(double th0, unsigned int id, unsigned int limit);
 };
 
